module dsp
    implicit none
    integer, parameter :: DSPERR_DOMAIN = 1

contains

    function theta(x) !Heaviside function, useful in DSP
        real :: theta
        real, intent(in) :: x

        theta = 1.0
        if (x < 0.0 ) theta = 0.0
    end function theta

    function sinc(x) !sinc function as used in DSP
        real :: sinc, xpi
        real, intent(in) :: x
        real, parameter :: pi = acos(-1.0)

        xpi = x*pi
        sinc = 1.0
        if (xpi /= 0.0) sinc = sin(xpi)/xpi
    end function sinc

    function rect(t, tau, info) !generalized rectangular function, useful in DSP
    real :: rect
        real, intent(in) :: t, tau
        integer, intent(out), optional :: info
        real :: abs_t, half_tau

        if (present(info)) info = 0

        if (tau <= 0.0) then
            if (present(info)) info = DSPERR_DOMAIN
            rect = 0.0
            return
        end if

        abs_t = abs(t)
        half_tau = 0.5*tau
        rect = 0.5
        if (abs_t /= half_tau) rect = theta(half_tau-abs_t)
    end function rect

end module dsp
